@@ -9,7 +9,16 @@ Contributed Code
     This code makes no guarantee about maintaining backwards compatibility.
 """
 
-from pyro.contrib import autoname, bnn, easyguide, epidemiology, forecast, gp, oed, tracking
+from pyro.contrib import (
+    autoname,
+    bnn,
+    easyguide,
+    epidemiology,
+    forecast,
+    gp,
+    oed,
+    tracking,
+)
 
 __all__ = [
     "autoname",
@@ -21,3 +30,13 @@ __all__ = [
     "oed",
     "tracking",
 ]
+
+
+try:
+    import funsor as funsor_  # noqa: F401
+
+    from pyro.contrib import funsor
+
+    __all__ += ["funsor"]
+except ImportError:
+    pass
