@@ -6,7 +6,7 @@ import urllib
 
 import pandas as pd
 
-from pyro.contrib.examples.util import get_data_directory, _mkdir_p
+from pyro.contrib.examples.util import _mkdir_p, get_data_directory
 
 DATA = get_data_directory(__file__)
 
