@@ -2,8 +2,9 @@
 # SPDX-License-Identifier: Apache-2.0
 
 import queue
-from pyro.infer.abstract_infer import TracePosterior
+
 import pyro.poutine as poutine
+from pyro.infer.abstract_infer import TracePosterior
 
 ###################################
 # Search borrowed from RSA example
@@ -14,6 +15,7 @@ class Search(TracePosterior):
     """
     Exact inference by enumerating over all possible executions
     """
+
     def __init__(self, model, max_tries=int(1e6), **kwargs):
         self.model = model
         self.max_tries = max_tries
@@ -22,8 +24,7 @@ class Search(TracePosterior):
     def _traces(self, *args, **kwargs):
         q = queue.Queue()
         q.put(poutine.Trace())
-        p = poutine.trace(
-            poutine.queue(self.model, queue=q, max_tries=self.max_tries))
+        p = poutine.trace(poutine.queue(self.model, queue=q, max_tries=self.max_tries))
         while not q.empty():
             tr = p.get_trace(*args, **kwargs)
             yield tr, tr.log_prob_sum()
