@@ -34,13 +34,15 @@ class CoalescentTimesConstraint(constraints.Constraint):
 
 class CoalescentTimes(TorchDistribution):
     """
-    Distribution over coalescent times given irregular sampled ``leaf_times``.
+    Distribution over sorted coalescent times given irregular sampled
+    ``leaf_times`` and constant population size.
 
-    Sample values will be sorted sets of binary coalescent times. Each sample
-    ``value`` will have cardinality ``value.size(-1) = leaf_times.size(-1) -
-    1``, so that phylogenies are complete binary trees.  This distribution can
-    thus be batched over multiple samples of phylogenies given fixed (number
-    of) leaf times, e.g. over phylogeny samples from BEAST or MrBayes.
+    Sample values will be **sorted** sets of binary coalescent times. Each
+    sample ``value`` will have cardinality ``value.size(-1) =
+    leaf_times.size(-1) - 1``, so that phylogenies are complete binary trees.
+    This distribution can thus be batched over multiple samples of phylogenies
+    given fixed (number of) leaf times, e.g. over phylogeny samples from BEAST
+    or MrBayes.
 
     **References**
 
@@ -54,13 +56,18 @@ class CoalescentTimes(TorchDistribution):
     :param torch.Tensor leaf_times: Vector of times of sampling events, i.e.
         leaf nodes in the phylogeny. These can be arbitrary real numbers with
         arbitrary order and duplicates.
+    :param torch.Tensor rate: Base coalescent rate (pairwise rate of
+        coalescence) under a constant population size model. Defaults to 1.
     """
-    arg_constraints = {"leaf_times": constraints.real}
 
-    def __init__(self, leaf_times, *, validate_args=None):
+    arg_constraints = {"leaf_times": constraints.real, "rate": constraints.positive}
+
+    def __init__(self, leaf_times, rate=1.0, *, validate_args=None):
+        rate = torch.as_tensor(rate, dtype=leaf_times.dtype, device=leaf_times.device)
+        batch_shape = broadcast_shape(rate.shape, leaf_times.shape[:-1])
         event_shape = (leaf_times.size(-1) - 1,)
-        batch_shape = leaf_times.shape[:-1]
         self.leaf_times = leaf_times
+        self.rate = rate
         super().__init__(batch_shape, event_shape, validate_args=validate_args)
 
     @constraints.dependent_property
@@ -77,7 +84,9 @@ class CoalescentTimes(TorchDistribution):
         # in the number of lineages, which changes at each event.
         binomial = phylogeny.binomial[..., :-1]
         interval = phylogeny.times[..., :-1] - phylogeny.times[..., 1:]
-        log_prob = -(binomial * interval).sum(-1)
+        log_prob = self.rate.log() * coal_times.size(-1) - self.rate * (
+            binomial * interval
+        ).sum(-1)
 
         # Scaling by those rates and accounting for log|jacobian|, the density
         # is that of a collection of independent Exponential intervals.
@@ -129,10 +138,13 @@ class CoalescentTimesWithRate(TorchDistribution):
     :param torch.Tensor rate_grid: Tensor of base coalescent rates (pairwise
         rate of coalescence). For example in a simple SIR model this might be
         ``beta S / I``. The rightmost dimension is time, and this tensor
-        represents a (batch of) rates that are piecwise constant in time.
+        represents a (batch of) rates that are piecewise constant in time.
     """
-    arg_constraints = {"leaf_times": constraints.real,
-                       "rate_grid": constraints.positive}
+
+    arg_constraints = {
+        "leaf_times": constraints.real,
+        "rate_grid": constraints.positive,
+    }
 
     def __init__(self, leaf_times, rate_grid, *, validate_args=None):
         batch_shape = broadcast_shape(leaf_times.shape[:-1], rate_grid.shape[:-1])
@@ -154,7 +166,8 @@ class CoalescentTimesWithRate(TorchDistribution):
         new.leaf_times = self.leaf_times
         new.rate_grid = self.rate_grid
         super(CoalescentTimesWithRate, new).__init__(
-            batch_shape, self.event_shape, validate_args=False)
+            batch_shape, self.event_shape, validate_args=False
+        )
         new._validate_args = self.__dict__.get("_validate_args")
         return new
 
@@ -166,9 +179,6 @@ class CoalescentTimesWithRate(TorchDistribution):
         number of time steps, ``N`` is the number of leaves, and ``S =
         sample_shape.numel()`` is the number of samples of ``value``.
 
-        This is differentiable wrt ``rate_grid`` but neither ``leaf_times`` nor
-        ``value = coal_times``.
-
         :param torch.Tensor value: A tensor of coalescent times. These denote
             sets of size ``leaf_times.size(-1) - 1`` along the trailing
             dimension and should be sorted along that dimension.
@@ -183,7 +193,9 @@ class CoalescentTimesWithRate(TorchDistribution):
         # Compute survival factors for closed intervals.
         cumsum = self.rate_grid.cumsum(-1)
         cumsum = torch.nn.functional.pad(cumsum, (1, 0), value=0)
-        integral = _interpolate_gather(cumsum, phylogeny.times[..., 1:])  # ignore the final lonely leaf
+        integral = _interpolate_gather(
+            cumsum, phylogeny.times[..., 1:]
+        )  # ignore the final lonely leaf
         integral = integral[..., :-1] - integral[..., 1:]
         integral = integral.clamp(min=torch.finfo(integral.dtype).tiny)  # avoid nan
         log_prob = -(phylogeny.binomial[..., 1:-1] * integral).sum(-1)
@@ -233,6 +245,7 @@ class CoalescentRateLikelihood:
         and should be sorted along that dimension.
     :param int duration: Size of the rate grid, ``rate_grid.size(-1)``.
     """
+
     def __init__(self, leaf_times, coal_times, duration, *, validate_args=None):
         assert leaf_times.size(-1) == 1 + coal_times.size(-1)
         assert isinstance(duration, int) and duration >= 2
@@ -253,9 +266,14 @@ class CoalescentRateLikelihood:
         times = phylogeny.times.clamp(min=duration)
         intervals = times[..., 1:] - times[..., :-1]
         post_linear = (phylogeny.binomial[..., :-1] * intervals).sum(-1, keepdim=True)
-        self._linear = torch.cat([pre_linear,
-                                  new_zeros(pre_linear.shape[:-1] + (duration - 2,)),
-                                  post_linear], dim=-1)
+        self._linear = torch.cat(
+            [
+                pre_linear,
+                new_zeros(pre_linear.shape[:-1] + (duration - 2,)),
+                post_linear,
+            ],
+            dim=-1,
+        )
 
         # Construct linear part from intervals of survival within [0, duration].
         times = phylogeny.times.clamp(min=0, max=duration)
@@ -298,7 +316,10 @@ class CoalescentRateLikelihood:
         """
         const = self._const[..., t]
         linear = self._linear[..., t] * rate_grid
-        log = self._log[..., t] * rate_grid.clamp(min=torch.finfo(rate_grid.dtype).tiny).log()
+        log = (
+            self._log[..., t]
+            * rate_grid.clamp(min=torch.finfo(rate_grid.dtype).tiny).log()
+        )
         return const + linear + log
 
 
@@ -322,6 +343,7 @@ def bio_phylo_to_times(tree, *, get_time=None):
         def get_branch_length(clade):
             branch_length = clade.branch_length
             return 1.0 if branch_length is None else branch_length
+
         times = {tree.root: get_branch_length(tree.root)}
 
     leaf_times = []
@@ -393,27 +415,40 @@ def _weak_memoize(fn):
     @functools.wraps(fn)
     def memoized_fn(*args):
         key = tuple(map(id, args))
-        if key not in cache:
-            cache[key] = fn(*args)
-            for arg in args:
-                weakref.finalize(arg, cache.pop, key, None)
-        return cache[key]
+
+        # Allow cache hit only when tensors have not since been mutated.
+        version = tuple(arg._version for arg in args)
+        if key in cache:
+            old_version, result = cache[key]
+            if old_version == version:
+                return result
+
+        result = fn(*args)
+        cache[key] = version, result
+        for arg in args:
+            weakref.finalize(arg, cache.pop, key, None)
+        return result
 
     return memoized_fn
 
 
 # This helper data structure has only timing information.
-_Phylogeny = namedtuple("_Phylogeny", (
-    "times", "signs", "lineages", "binomial", "coal_binomial",
-))
+_Phylogeny = namedtuple(
+    "_Phylogeny",
+    (
+        "times",
+        "signs",
+        "lineages",
+        "binomial",
+        "coal_binomial",
+    ),
+)
 
 
 @_weak_memoize
 @torch.no_grad()
 def _make_phylogeny(leaf_times, coal_times):
     assert leaf_times.size(-1) == 1 + coal_times.size(-1)
-    assert not leaf_times.requires_grad
-    assert not coal_times.requires_grad
 
     # Expand shapes to match.
     N = leaf_times.size(-1)
@@ -427,7 +462,9 @@ def _make_phylogeny(leaf_times, coal_times):
     # (coal_times) into a pair (times, signs) of arrays of length 2N-1, where
     # leaf sample sign is +1 and coalescent sign is -1.
     times = torch.cat([coal_times, leaf_times], dim=-1)
-    signs = torch.linspace(1.5 - N, N - 0.5, 2 * N - 1).sign()  # e.g. [-1, -1, +1, +1, +1]
+    signs = torch.linspace(
+        1.5 - N, N - 0.5, 2 * N - 1
+    ).sign()  # e.g. [-1, -1, +1, +1, +1]
 
     # Sort the events reverse-ordered in time, i.e. latest to earliest.
     times, index = times.sort(dim=-1, descending=True)
@@ -441,7 +478,7 @@ def _make_phylogeny(leaf_times, coal_times):
     binomial = lineages * (lineages - 1) / 2
 
     # Compute the binomial coefficient following each coalescent event.
-    coal_index = inv_index[..., :N - 1]
+    coal_index = inv_index[..., : N - 1]
     coal_binomial = binomial.gather(-1, coal_index - 1)
 
     return _Phylogeny(times, signs, lineages, binomial, coal_binomial)
@@ -449,6 +486,7 @@ def _make_phylogeny(leaf_times, coal_times):
 
 def _sample_coalescent_times(leaf_times):
     leaf_times = leaf_times.detach()
+    proto = leaf_times
     N = leaf_times.size(-1)
     batch_shape = leaf_times.shape[:-1]
 
@@ -456,7 +494,9 @@ def _sample_coalescent_times(leaf_times):
     # instead we simply sequentially sample and stack.
     if batch_shape:
         flat_leaf_times = leaf_times.reshape(-1, N)
-        flat_coal_times = torch.stack(list(map(_sample_coalescent_times, flat_leaf_times)))
+        flat_coal_times = torch.stack(
+            list(map(_sample_coalescent_times, flat_leaf_times))
+        )
         return flat_coal_times.reshape(batch_shape + (N - 1,))
     assert leaf_times.shape == (N,)
 
@@ -468,7 +508,7 @@ def _sample_coalescent_times(leaf_times):
     t = leaf_times[leaf]
     active = 2
     binomial = active * (active - 1) / 2
-    for u in torch.empty(N - 1).exponential_().tolist():
+    for u in proto.new_empty(N - 1).exponential_().tolist():
         while leaf + 1 < N and u > (t - leaf_times[leaf + 1]) * binomial:
             # Move past the next leaf.
             leaf += 1
@@ -483,4 +523,4 @@ def _sample_coalescent_times(leaf_times):
         coal_times.append(t)
     coal_times.reverse()
 
-    return torch.tensor(coal_times)
+    return proto.new_tensor(coal_times)
