@@ -2,19 +2,29 @@
 # SPDX-License-Identifier: Apache-2.0
 
 import torch
-from torch.distributions import constraints
 
-from pyro.distributions.torch import Categorical, Gamma, Independent, MultivariateNormal
-from pyro.distributions.torch_distribution import TorchDistribution
-from pyro.distributions.util import broadcast_shape
-from pyro.ops.gamma_gaussian import (GammaGaussian, gamma_and_mvn_to_gamma_gaussian, gamma_gaussian_tensordot,
-                                     matrix_and_mvn_to_gamma_gaussian)
-from pyro.ops.gaussian import Gaussian, gaussian_tensordot, matrix_and_mvn_to_gaussian, mvn_to_gaussian
+from pyro.ops.gamma_gaussian import (
+    GammaGaussian,
+    gamma_and_mvn_to_gamma_gaussian,
+    gamma_gaussian_tensordot,
+    matrix_and_mvn_to_gamma_gaussian,
+)
+from pyro.ops.gaussian import (
+    Gaussian,
+    gaussian_tensordot,
+    matrix_and_mvn_to_gaussian,
+    mvn_to_gaussian,
+)
 from pyro.ops.special import safe_log
 from pyro.ops.tensor_utils import cholesky, cholesky_solve
 
+from . import constraints
+from .torch import Categorical, Gamma, Independent, MultivariateNormal
+from .torch_distribution import TorchDistribution
+from .util import broadcast_shape, torch_jit_script_if_tracing
 
-@torch.jit.script
+
+@torch_jit_script_if_tracing
 def _linear_integrate(init, trans, shift):
     """
     Integrate the inhomogeneous linear shifterence equation::
@@ -39,15 +49,15 @@ def _logmatmulexp(x, y):
     Numerically stable version of ``(x.log() @ y.log()).exp()``.
     """
     finfo = torch.finfo(x.dtype)  # avoid nan due to -inf - -inf
-    x_shift = x.max(-1, keepdim=True).values.clamp(min=finfo.min)
-    y_shift = y.max(-2, keepdim=True).values.clamp(min=finfo.min)
+    x_shift = x.detach().max(-1, keepdim=True).values.clamp_(min=finfo.min)
+    y_shift = y.detach().max(-2, keepdim=True).values.clamp_(min=finfo.min)
     xy = safe_log(torch.matmul((x - x_shift).exp(), (y - y_shift).exp()))
     return xy + x_shift + y_shift
 
 
 # TODO re-enable jitting once _SafeLog is supported by the jit.
 # See https://discuss.pytorch.org/t/does-torch-jit-script-support-custom-operators/65759/4
-# @torch.jit.script
+# @torch_jit_script_if_tracing
 def _sequential_logmatmulexp(logits):
     """
     For a tensor ``x`` whose time dimension is -3, computes::
@@ -109,9 +119,13 @@ def _sequential_gaussian_filter_sample(init, trans, sample_shape):
     assert _is_subshape(trans.batch_shape[:-1], init.batch_shape)
     state_dim = trans.dim() // 2
     device = trans.precision.device
-    perm = torch.cat([torch.arange(1 * state_dim, 2 * state_dim, device=device),
-                      torch.arange(0 * state_dim, 1 * state_dim, device=device),
-                      torch.arange(2 * state_dim, 3 * state_dim, device=device)])
+    perm = torch.cat(
+        [
+            torch.arange(1 * state_dim, 2 * state_dim, device=device),
+            torch.arange(0 * state_dim, 1 * state_dim, device=device),
+            torch.arange(2 * state_dim, 3 * state_dim, device=device),
+        ]
+    )
 
     # Forward filter, similar to _sequential_gaussian_tensordot().
     tape = []
@@ -147,10 +161,13 @@ def _sequential_gaussian_filter_sample(init, trans, sample_shape):
             cond = cond.reshape(shape + (-1, 2 * state_dim))  # [z0z2, z2z4]
             sample = joint.condition(cond).rsample()  # [z1, z3]
             sample = torch.nn.functional.pad(sample, (0, 0, 0, 1))  # [z1, z3, 0]
-            result = torch.stack([
-                result,  # [z0, z2, z4]
-                sample,  # [z1, z3, 0]
-            ], dim=-2)  # [[z0, z1], [z2, z3], [z4, 0]]
+            result = torch.stack(
+                [
+                    result,  # [z0, z2, z4]
+                    sample,  # [z1, z3, 0]
+                ],
+                dim=-2,
+            )  # [[z0, z1], [z2, z3], [z4, 0]]
             result = result.reshape(shape + (-1, state_dim))  # [z0, z1, z2, z3, z4, 0]
             result = result[..., :-1, :]  # [z0, z1, z2, z3, z4]
         else:  # ODD case.
@@ -161,10 +178,13 @@ def _sequential_gaussian_filter_sample(init, trans, sample_shape):
             cond = cond.reshape(shape + (-1, 2 * state_dim))  # [z0z2]
             sample = joint.condition(cond).rsample()  # [z1]
             sample = torch.cat([sample, result[..., -1:, :]], dim=-2)  # [z1, z3]
-            result = torch.stack([
-                result[..., :-1, :],  # [z0, z2]
-                sample,  # [z1, z3]
-            ], dim=-2)  # [[z0, z1], [z2, z3]]
+            result = torch.stack(
+                [
+                    result[..., :-1, :],  # [z0, z2]
+                    sample,  # [z1, z3]
+                ],
+                dim=-2,
+            )  # [[z0, z1], [z2, z3]]
             result = result.reshape(shape + (-1, state_dim))  # [z0, z1, z2, z3]
 
     return result[..., 1:, :]  # [z1, z2, z3, ...]
@@ -188,7 +208,9 @@ def _sequential_gamma_gaussian_tensordot(gamma_gaussian):
         x, y = x_y[..., 0], x_y[..., 1]
         contracted = gamma_gaussian_tensordot(x, y, state_dim)
         if time > even_time:
-            contracted = GammaGaussian.cat((contracted, gamma_gaussian[..., -1:]), dim=-1)
+            contracted = GammaGaussian.cat(
+                (contracted, gamma_gaussian[..., -1:]), dim=-1
+            )
         gamma_gaussian = contracted
     return gamma_gaussian[..., 0]
 
@@ -203,6 +225,7 @@ class HiddenMarkovModel(TorchDistribution):
         This is required when sampling from homogeneous HMMs whose parameters
         are not expanded along the time axis.
     """
+
     def __init__(self, duration, batch_shape, event_shape, validate_args=None):
         if duration is None:
             if event_shape[0] != 1:
@@ -210,8 +233,11 @@ class HiddenMarkovModel(TorchDistribution):
                 duration = event_shape[0]
         elif duration != event_shape[0]:
             if event_shape[0] != 1:
-                raise ValueError("duration, event_shape mismatch: {} vs {}"
-                                 .format(duration, event_shape))
+                raise ValueError(
+                    "duration, event_shape mismatch: {} vs {}".format(
+                        duration, event_shape
+                    )
+                )
             # Infer event_shape from duration.
             event_shape = torch.Size((duration,) + event_shape[1:])
         self._duration = duration
@@ -285,33 +311,54 @@ class DiscreteHMM(HiddenMarkovModel):
         This is required when sampling from homogeneous HMMs whose parameters
         are not expanded along the time axis.
     """
-    arg_constraints = {"initial_logits": constraints.real,
-                       "transition_logits": constraints.real}
 
-    def __init__(self, initial_logits, transition_logits, observation_dist,
-                 validate_args=None, duration=None):
+    arg_constraints = {
+        "initial_logits": constraints.real,
+        "transition_logits": constraints.real,
+    }
+
+    def __init__(
+        self,
+        initial_logits,
+        transition_logits,
+        observation_dist,
+        validate_args=None,
+        duration=None,
+    ):
         if initial_logits.dim() < 1:
-            raise ValueError("expected initial_logits to have at least one dim, "
-                             "actual shape = {}".format(initial_logits.shape))
+            raise ValueError(
+                "expected initial_logits to have at least one dim, "
+                "actual shape = {}".format(initial_logits.shape)
+            )
         if transition_logits.dim() < 2:
-            raise ValueError("expected transition_logits to have at least two dims, "
-                             "actual shape = {}".format(transition_logits.shape))
+            raise ValueError(
+                "expected transition_logits to have at least two dims, "
+                "actual shape = {}".format(transition_logits.shape)
+            )
         if len(observation_dist.batch_shape) < 1:
-            raise ValueError("expected observation_dist to have at least one batch dim, "
-                             "actual .batch_shape = {}".format(observation_dist.batch_shape))
-        shape = broadcast_shape(initial_logits.shape[:-1] + (1,),
-                                transition_logits.shape[:-2],
-                                observation_dist.batch_shape[:-1])
+            raise ValueError(
+                "expected observation_dist to have at least one batch dim, "
+                "actual .batch_shape = {}".format(observation_dist.batch_shape)
+            )
+        shape = broadcast_shape(
+            initial_logits.shape[:-1] + (1,),
+            transition_logits.shape[:-2],
+            observation_dist.batch_shape[:-1],
+        )
         batch_shape, time_shape = shape[:-1], shape[-1:]
         event_shape = time_shape + observation_dist.event_shape
         self.initial_logits = initial_logits - initial_logits.logsumexp(-1, True)
-        self.transition_logits = transition_logits - transition_logits.logsumexp(-1, True)
+        self.transition_logits = transition_logits - transition_logits.logsumexp(
+            -1, True
+        )
         self.observation_dist = observation_dist
-        super().__init__(duration, batch_shape, event_shape, validate_args=validate_args)
+        super().__init__(
+            duration, batch_shape, event_shape, validate_args=validate_args
+        )
 
-    @property
+    @constraints.dependent_property(event_dim=2)
     def support(self):
-        return self.observation_dist.support
+        return constraints.independent(self.observation_dist.support, 1)
 
     def expand(self, batch_shape, _instance=None):
         new = self._get_checked_instance(DiscreteHMM, _instance)
@@ -323,8 +370,10 @@ class DiscreteHMM(HiddenMarkovModel):
         new.initial_logits = self.initial_logits.expand(batch_shape + (-1,))
         new.transition_logits = self.transition_logits
         new.observation_dist = self.observation_dist
-        super(DiscreteHMM, new).__init__(self.duration, batch_shape, self.event_shape, validate_args=False)
-        new._validate_args = self.__dict__.get('_validate_args')
+        super(DiscreteHMM, new).__init__(
+            self.duration, batch_shape, self.event_shape, validate_args=False
+        )
+        new._validate_args = self.__dict__.get("_validate_args")
         return new
 
     def log_prob(self, value):
@@ -434,36 +483,52 @@ class GaussianHMM(HiddenMarkovModel):
         This is required when sampling from homogeneous HMMs whose parameters
         are not expanded along the time axis.
     """
+
     has_rsample = True
     arg_constraints = {}
-    support = constraints.real
+    support = constraints.independent(constraints.real, 2)
 
-    def __init__(self, initial_dist, transition_matrix, transition_dist,
-                 observation_matrix, observation_dist, validate_args=None, duration=None):
-        assert (isinstance(initial_dist, torch.distributions.MultivariateNormal) or
-                (isinstance(initial_dist, torch.distributions.Independent) and
-                 isinstance(initial_dist.base_dist, torch.distributions.Normal)))
+    def __init__(
+        self,
+        initial_dist,
+        transition_matrix,
+        transition_dist,
+        observation_matrix,
+        observation_dist,
+        validate_args=None,
+        duration=None,
+    ):
+        assert isinstance(initial_dist, torch.distributions.MultivariateNormal) or (
+            isinstance(initial_dist, torch.distributions.Independent)
+            and isinstance(initial_dist.base_dist, torch.distributions.Normal)
+        )
         assert isinstance(transition_matrix, torch.Tensor)
-        assert (isinstance(transition_dist, torch.distributions.MultivariateNormal) or
-                (isinstance(transition_dist, torch.distributions.Independent) and
-                 isinstance(transition_dist.base_dist, torch.distributions.Normal)))
+        assert isinstance(transition_dist, torch.distributions.MultivariateNormal) or (
+            isinstance(transition_dist, torch.distributions.Independent)
+            and isinstance(transition_dist.base_dist, torch.distributions.Normal)
+        )
         assert isinstance(observation_matrix, torch.Tensor)
-        assert (isinstance(observation_dist, torch.distributions.MultivariateNormal) or
-                (isinstance(observation_dist, torch.distributions.Independent) and
-                 isinstance(observation_dist.base_dist, torch.distributions.Normal)))
+        assert isinstance(observation_dist, torch.distributions.MultivariateNormal) or (
+            isinstance(observation_dist, torch.distributions.Independent)
+            and isinstance(observation_dist.base_dist, torch.distributions.Normal)
+        )
         hidden_dim, obs_dim = observation_matrix.shape[-2:]
         assert initial_dist.event_shape == (hidden_dim,)
         assert transition_matrix.shape[-2:] == (hidden_dim, hidden_dim)
         assert transition_dist.event_shape == (hidden_dim,)
         assert observation_dist.event_shape == (obs_dim,)
-        shape = broadcast_shape(initial_dist.batch_shape + (1,),
-                                transition_matrix.shape[:-2],
-                                transition_dist.batch_shape,
-                                observation_matrix.shape[:-2],
-                                observation_dist.batch_shape)
+        shape = broadcast_shape(
+            initial_dist.batch_shape + (1,),
+            transition_matrix.shape[:-2],
+            transition_dist.batch_shape,
+            observation_matrix.shape[:-2],
+            observation_dist.batch_shape,
+        )
         batch_shape, time_shape = shape[:-1], shape[-1:]
         event_shape = time_shape + (obs_dim,)
-        super().__init__(duration, batch_shape, event_shape, validate_args=validate_args)
+        super().__init__(
+            duration, batch_shape, event_shape, validate_args=validate_args
+        )
 
         self.hidden_dim = hidden_dim
         self.obs_dim = obs_dim
@@ -484,8 +549,10 @@ class GaussianHMM(HiddenMarkovModel):
         batch_shape = torch.Size(broadcast_shape(self.batch_shape, batch_shape))
         new._init = self._init.expand(batch_shape)
 
-        super(GaussianHMM, new).__init__(self.duration, batch_shape, self.event_shape, validate_args=False)
-        new._validate_args = self.__dict__.get('_validate_args')
+        super(GaussianHMM, new).__init__(
+            self.duration, batch_shape, self.event_shape, validate_args=False
+        )
+        new._validate_args = self.__dict__.get("_validate_args")
         return new
 
     def log_prob(self, value):
@@ -493,7 +560,9 @@ class GaussianHMM(HiddenMarkovModel):
             self._validate_sample(value)
 
         # Combine observation and transition factors.
-        result = self._trans + self._obs.condition(value).event_pad(left=self.hidden_dim)
+        result = self._trans + self._obs.condition(value).event_pad(
+            left=self.hidden_dim
+        )
 
         # Eliminate time dimension.
         result = _sequential_gaussian_tensordot(result.expand(result.batch_shape))
@@ -508,7 +577,9 @@ class GaussianHMM(HiddenMarkovModel):
     def rsample(self, sample_shape=torch.Size()):
         assert self.duration is not None
         sample_shape = torch.Size(sample_shape)
-        trans = self._trans + self._obs.marginalize(right=self.obs_dim).event_pad(left=self.hidden_dim)
+        trans = self._trans + self._obs.marginalize(right=self.obs_dim).event_pad(
+            left=self.hidden_dim
+        )
         trans = trans.expand(trans.batch_shape[:-1] + (self.duration,))
         z = _sequential_gaussian_filter_sample(self._init, trans, sample_shape)
         x = self._obs.left_condition(z).rsample()
@@ -548,9 +619,12 @@ class GaussianHMM(HiddenMarkovModel):
 
         # Convert to a distribution
         precision = logp.precision
-        loc = cholesky_solve(logp.info_vec.unsqueeze(-1), cholesky(precision)).squeeze(-1)
-        return MultivariateNormal(loc, precision_matrix=precision,
-                                  validate_args=self._validate_args)
+        loc = cholesky_solve(logp.info_vec.unsqueeze(-1), cholesky(precision)).squeeze(
+            -1
+        )
+        return MultivariateNormal(
+            loc, precision_matrix=precision, validate_args=self._validate_args
+        )
 
     def conjugate_update(self, other):
         """
@@ -570,9 +644,10 @@ class GaussianHMM(HiddenMarkovModel):
             updated :class:`GaussianHMM` , and ``log_normalizer`` is a
             :class:`~torch.Tensor` representing the normalization factor.
         """
-        assert (isinstance(other, torch.distributions.Independent) and
-                (isinstance(other.base_dist, torch.distributions.Normal) or
-                 isinstance(other.base_dist, torch.distributions.MultivariateNormal)))
+        assert isinstance(other, torch.distributions.Independent) and (
+            isinstance(other.base_dist, torch.distributions.Normal)
+            or isinstance(other.base_dist, torch.distributions.MultivariateNormal)
+        )
         duration = other.event_shape[0] if self.duration is None else self.duration
         event_shape = torch.Size((duration, self.obs_dim))
         assert other.event_shape == event_shape
@@ -582,19 +657,25 @@ class GaussianHMM(HiddenMarkovModel):
         new.obs_dim = self.obs_dim
         new._init = self._init
         new._trans = self._trans
-        new._obs = self._obs + mvn_to_gaussian(other.to_event(-1)).event_pad(left=self.hidden_dim)
+        new._obs = self._obs + mvn_to_gaussian(other.to_event(-1)).event_pad(
+            left=self.hidden_dim
+        )
 
         # Normalize.
         # TODO cache this computation for the forward pass of .rsample().
-        logp = new._trans + new._obs.marginalize(right=new.obs_dim).event_pad(left=new.hidden_dim)
+        logp = new._trans + new._obs.marginalize(right=new.obs_dim).event_pad(
+            left=new.hidden_dim
+        )
         logp = _sequential_gaussian_tensordot(logp.expand(logp.batch_shape))
         logp = gaussian_tensordot(new._init, logp, dims=new.hidden_dim)
         log_normalizer = logp.event_logsumexp()
         new._init = new._init - log_normalizer
 
         batch_shape = log_normalizer.shape
-        super(GaussianHMM, new).__init__(duration, batch_shape, event_shape, validate_args=False)
-        new._validate_args = self.__dict__.get('_validate_args')
+        super(GaussianHMM, new).__init__(
+            duration, batch_shape, event_shape, validate_args=False
+        )
+        new._validate_args = self.__dict__.get("_validate_args")
         return new, log_normalizer
 
     def prefix_condition(self, data):
@@ -630,20 +711,24 @@ class GaussianHMM(HiddenMarkovModel):
             left._obs = self._obs[..., :t]
             right._obs = self._obs[..., t:]
 
-        if self._trans.batch_shape == () or self._trans.batch_shape[-1] == 1:  # homogeneous
+        if (
+            self._trans.batch_shape == () or self._trans.batch_shape[-1] == 1
+        ):  # homogeneous
             left._trans = self._trans
             right._trans = self._trans
         else:  # heterogeneous
             left._trans = self._trans[..., :t]
             right._trans = self._trans[..., t:]
 
-        super(GaussianHMM, left).__init__(t, self.batch_shape, (t, self.obs_dim),
-                                          validate_args=self._validate_args)
+        super(GaussianHMM, left).__init__(
+            t, self.batch_shape, (t, self.obs_dim), validate_args=self._validate_args
+        )
         initial_dist = left.filter(data)
         right._init = mvn_to_gaussian(initial_dist)
         batch_shape = broadcast_shape(right._init.batch_shape, self.batch_shape)
-        super(GaussianHMM, right).__init__(f, batch_shape, (f, self.obs_dim),
-                                           validate_args=self._validate_args)
+        super(GaussianHMM, right).__init__(
+            f, batch_shape, (f, self.obs_dim), validate_args=self._validate_args
+        )
         return right
 
 
@@ -716,11 +801,21 @@ class GammaGaussianHMM(HiddenMarkovModel):
         This is required when sampling from homogeneous HMMs whose parameters
         are not expanded along the time axis.
     """
-    arg_constraints = {}
-    support = constraints.real
 
-    def __init__(self, scale_dist, initial_dist, transition_matrix, transition_dist,
-                 observation_matrix, observation_dist, validate_args=None, duration=None):
+    arg_constraints = {}
+    support = constraints.independent(constraints.real, 2)
+
+    def __init__(
+        self,
+        scale_dist,
+        initial_dist,
+        transition_matrix,
+        transition_dist,
+        observation_matrix,
+        observation_dist,
+        validate_args=None,
+        duration=None,
+    ):
         assert isinstance(scale_dist, Gamma)
         assert isinstance(initial_dist, MultivariateNormal)
         assert isinstance(transition_matrix, torch.Tensor)
@@ -732,20 +827,28 @@ class GammaGaussianHMM(HiddenMarkovModel):
         assert transition_matrix.shape[-2:] == (hidden_dim, hidden_dim)
         assert transition_dist.event_shape == (hidden_dim,)
         assert observation_dist.event_shape == (obs_dim,)
-        shape = broadcast_shape(scale_dist.batch_shape + (1,),
-                                initial_dist.batch_shape + (1,),
-                                transition_matrix.shape[:-2],
-                                transition_dist.batch_shape,
-                                observation_matrix.shape[:-2],
-                                observation_dist.batch_shape)
+        shape = broadcast_shape(
+            scale_dist.batch_shape + (1,),
+            initial_dist.batch_shape + (1,),
+            transition_matrix.shape[:-2],
+            transition_dist.batch_shape,
+            observation_matrix.shape[:-2],
+            observation_dist.batch_shape,
+        )
         batch_shape, time_shape = shape[:-1], shape[-1:]
         event_shape = time_shape + (obs_dim,)
-        super().__init__(duration, batch_shape, event_shape, validate_args=validate_args)
+        super().__init__(
+            duration, batch_shape, event_shape, validate_args=validate_args
+        )
         self.hidden_dim = hidden_dim
         self.obs_dim = obs_dim
         self._init = gamma_and_mvn_to_gamma_gaussian(scale_dist, initial_dist)
-        self._trans = matrix_and_mvn_to_gamma_gaussian(transition_matrix, transition_dist)
-        self._obs = matrix_and_mvn_to_gamma_gaussian(observation_matrix, observation_dist)
+        self._trans = matrix_and_mvn_to_gamma_gaussian(
+            transition_matrix, transition_dist
+        )
+        self._obs = matrix_and_mvn_to_gamma_gaussian(
+            observation_matrix, observation_dist
+        )
 
     def expand(self, batch_shape, _instance=None):
         new = self._get_checked_instance(GammaGaussianHMM, _instance)
@@ -758,8 +861,10 @@ class GammaGaussianHMM(HiddenMarkovModel):
         new._init = self._init.expand(batch_shape)
         new._trans = self._trans
         new._obs = self._obs
-        super(GammaGaussianHMM, new).__init__(self.duration, batch_shape, self.event_shape, validate_args=False)
-        new._validate_args = self.__dict__.get('_validate_args')
+        super(GammaGaussianHMM, new).__init__(
+            self.duration, batch_shape, self.event_shape, validate_args=False
+        )
+        new._validate_args = self.__dict__.get("_validate_args")
         return new
 
     def log_prob(self, value):
@@ -767,7 +872,9 @@ class GammaGaussianHMM(HiddenMarkovModel):
             self._validate_sample(value)
 
         # Combine observation and transition factors.
-        result = self._trans + self._obs.condition(value).event_pad(left=self.hidden_dim)
+        result = self._trans + self._obs.condition(value).event_pad(
+            left=self.hidden_dim
+        )
 
         # Eliminate time dimension.
         result = _sequential_gamma_gaussian_tensordot(result.expand(result.batch_shape))
@@ -808,13 +915,15 @@ class GammaGaussianHMM(HiddenMarkovModel):
 
         # Posterior of the scale
         gamma_dist = logp.event_logsumexp()
-        scale_post = Gamma(gamma_dist.concentration, gamma_dist.rate,
-                           validate_args=self._validate_args)
+        scale_post = Gamma(
+            gamma_dist.concentration, gamma_dist.rate, validate_args=self._validate_args
+        )
         # Conditional of last state on unit scale
         scale_tril = cholesky(logp.precision)
         loc = cholesky_solve(logp.info_vec.unsqueeze(-1), scale_tril).squeeze(-1)
-        mvn = MultivariateNormal(loc, scale_tril=scale_tril,
-                                 validate_args=self._validate_args)
+        mvn = MultivariateNormal(
+            loc, scale_tril=scale_tril, validate_args=self._validate_args
+        )
         return scale_post, mvn
 
 
@@ -886,20 +995,30 @@ class LinearHMM(HiddenMarkovModel):
         are not expanded along the time axis.
     """
     arg_constraints = {}
-    support = constraints.real
+    support = constraints.independent(constraints.real, 2)
     has_rsample = True
 
-    def __init__(self, initial_dist, transition_matrix, transition_dist,
-                 observation_matrix, observation_dist,
-                 validate_args=None, duration=None):
+    def __init__(
+        self,
+        initial_dist,
+        transition_matrix,
+        transition_dist,
+        observation_matrix,
+        observation_dist,
+        validate_args=None,
+        duration=None,
+    ):
         assert initial_dist.has_rsample
         assert initial_dist.event_dim == 1
-        assert (isinstance(transition_matrix, torch.Tensor) and
-                transition_matrix.dim() >= 2)
+        assert (
+            isinstance(transition_matrix, torch.Tensor) and transition_matrix.dim() >= 2
+        )
         assert transition_dist.has_rsample
         assert transition_dist.event_dim == 1
-        assert (isinstance(observation_matrix, torch.Tensor) and
-                observation_matrix.dim() >= 2)
+        assert (
+            isinstance(observation_matrix, torch.Tensor)
+            and observation_matrix.dim() >= 2
+        )
         assert observation_dist.has_rsample
         assert observation_dist.event_dim == 1
 
@@ -908,26 +1027,32 @@ class LinearHMM(HiddenMarkovModel):
         assert transition_matrix.shape[-2:] == (hidden_dim, hidden_dim)
         assert transition_dist.event_shape == (hidden_dim,)
         assert observation_dist.event_shape == (obs_dim,)
-        shape = broadcast_shape(initial_dist.batch_shape + (1,),
-                                transition_matrix.shape[:-2],
-                                transition_dist.batch_shape,
-                                observation_matrix.shape[:-2],
-                                observation_dist.batch_shape)
+        shape = broadcast_shape(
+            initial_dist.batch_shape + (1,),
+            transition_matrix.shape[:-2],
+            transition_dist.batch_shape,
+            observation_matrix.shape[:-2],
+            observation_dist.batch_shape,
+        )
         batch_shape, time_shape = shape[:-1], shape[-1:]
         event_shape = time_shape + (obs_dim,)
-        super().__init__(duration, batch_shape, event_shape, validate_args=validate_args)
+        super().__init__(
+            duration, batch_shape, event_shape, validate_args=validate_args
+        )
 
         # Expand eagerly.
         if initial_dist.batch_shape != batch_shape:
             initial_dist = initial_dist.expand(batch_shape)
         if transition_matrix.shape[:-2] != batch_shape + time_shape:
             transition_matrix = transition_matrix.expand(
-                batch_shape + time_shape + (hidden_dim, hidden_dim))
+                batch_shape + time_shape + (hidden_dim, hidden_dim)
+            )
         if transition_dist.batch_shape != batch_shape + time_shape:
             transition_dist = transition_dist.expand(batch_shape + time_shape)
         if observation_matrix.shape[:-2] != batch_shape + time_shape:
             observation_matrix = observation_matrix.expand(
-                batch_shape + time_shape + (hidden_dim, obs_dim))
+                batch_shape + time_shape + (hidden_dim, obs_dim)
+            )
         if observation_dist.batch_shape != batch_shape + time_shape:
             observation_dist = observation_dist.expand(batch_shape + time_shape)
 
@@ -936,7 +1061,9 @@ class LinearHMM(HiddenMarkovModel):
         while True:
             if isinstance(observation_dist, torch.distributions.Independent):
                 observation_dist = observation_dist.base_dist
-            elif isinstance(observation_dist, torch.distributions.TransformedDistribution):
+            elif isinstance(
+                observation_dist, torch.distributions.TransformedDistribution
+            ):
                 transforms = observation_dist.transforms + transforms
                 observation_dist = observation_dist.base_dist
             else:
@@ -953,9 +1080,9 @@ class LinearHMM(HiddenMarkovModel):
         self.observation_dist = observation_dist
         self.transforms = transforms
 
-    @property
+    @constraints.dependent_property(event_dim=2)
     def support(self):
-        return self.observation_dist.support
+        return constraints.independent(self.observation_dist.support, 1)
 
     def expand(self, batch_shape, _instance=None):
         new = self._get_checked_instance(LinearHMM, _instance)
@@ -965,14 +1092,18 @@ class LinearHMM(HiddenMarkovModel):
         new.obs_dim = self.obs_dim
         new.initial_dist = self.initial_dist.expand(batch_shape)
         new.transition_matrix = self.transition_matrix.expand(
-            batch_shape + time_shape + (self.hidden_dim, self.hidden_dim))
+            batch_shape + time_shape + (self.hidden_dim, self.hidden_dim)
+        )
         new.transition_dist = self.transition_dist.expand(batch_shape + time_shape)
         new.observation_matrix = self.observation_matrix.expand(
-            batch_shape + time_shape + (self.hidden_dim, self.obs_dim))
+            batch_shape + time_shape + (self.hidden_dim, self.obs_dim)
+        )
         new.observation_dist = self.observation_dist.expand(batch_shape + time_shape)
         new.transforms = self.transforms
-        super(LinearHMM, new).__init__(self.duration, batch_shape, self.event_shape, validate_args=False)
-        new._validate_args = self.__dict__.get('_validate_args')
+        super(LinearHMM, new).__init__(
+            self.duration, batch_shape, self.event_shape, validate_args=False
+        )
+        new._validate_args = self.__dict__.get("_validate_args")
         return new
 
     def log_prob(self, value):
@@ -981,9 +1112,15 @@ class LinearHMM(HiddenMarkovModel):
     def rsample(self, sample_shape=torch.Size()):
         assert self.duration is not None
         init = self.initial_dist.rsample(sample_shape)
-        trans = self.transition_dist.expand(self.batch_shape + (self.duration,)).rsample(sample_shape)
-        obs = self.observation_dist.expand(self.batch_shape + (self.duration,)).rsample(sample_shape)
-        trans_matrix = self.transition_matrix.expand(self.batch_shape + (self.duration, -1, -1))
+        trans = self.transition_dist.expand(
+            self.batch_shape + (self.duration,)
+        ).rsample(sample_shape)
+        obs = self.observation_dist.expand(self.batch_shape + (self.duration,)).rsample(
+            sample_shape
+        )
+        trans_matrix = self.transition_matrix.expand(
+            self.batch_shape + (self.duration, -1, -1)
+        )
         z = _linear_integrate(init, trans_matrix, trans)
         x = (z.unsqueeze(-2) @ self.observation_matrix).squeeze(-2) + obs
         for t in self.transforms:
@@ -1006,6 +1143,7 @@ class IndependentHMM(TorchDistribution):
 
     :param HiddenMarkovModel base_dist: A base hidden Markov model instance.
     """
+
     arg_constraints = {}
 
     def __init__(self, base_dist):
@@ -1017,7 +1155,7 @@ class IndependentHMM(TorchDistribution):
         super().__init__(batch_shape, event_shape)
         self.base_dist = base_dist
 
-    @constraints.dependent_property
+    @constraints.dependent_property(event_dim=2)
     def support(self):
         return self.base_dist.support
 
@@ -1032,9 +1170,13 @@ class IndependentHMM(TorchDistribution):
     def expand(self, batch_shape, _instance=None):
         batch_shape = torch.Size(batch_shape)
         new = self._get_checked_instance(IndependentHMM, _instance)
-        new.base_dist = self.base_dist.expand(batch_shape + self.base_dist.batch_shape[-1:])
-        super(IndependentHMM, new).__init__(batch_shape, self.event_shape, validate_args=False)
-        new._validate_args = self.__dict__.get('_validate_args')
+        new.base_dist = self.base_dist.expand(
+            batch_shape + self.base_dist.batch_shape[-1:]
+        )
+        super(IndependentHMM, new).__init__(
+            batch_shape, self.event_shape, validate_args=False
+        )
+        new._validate_args = self.__dict__.get("_validate_args")
         return new
 
     def rsample(self, sample_shape=torch.Size()):
@@ -1086,9 +1228,12 @@ class GaussianMRF(TorchDistribution):
         have batch_shape broadcastable to ``self.batch_shape + (num_steps,)``.
         This should have event_shape ``(hidden_dim + obs_dim,)``.
     """
+
     arg_constraints = {}
 
-    def __init__(self, initial_dist, transition_dist, observation_dist, validate_args=None):
+    def __init__(
+        self, initial_dist, transition_dist, observation_dist, validate_args=None
+    ):
         assert isinstance(initial_dist, torch.distributions.MultivariateNormal)
         assert isinstance(transition_dist, torch.distributions.MultivariateNormal)
         assert isinstance(observation_dist, torch.distributions.MultivariateNormal)
@@ -1096,9 +1241,11 @@ class GaussianMRF(TorchDistribution):
         assert transition_dist.event_shape[0] == hidden_dim + hidden_dim
         obs_dim = observation_dist.event_shape[0] - hidden_dim
 
-        shape = broadcast_shape(initial_dist.batch_shape + (1,),
-                                transition_dist.batch_shape,
-                                observation_dist.batch_shape)
+        shape = broadcast_shape(
+            initial_dist.batch_shape + (1,),
+            transition_dist.batch_shape,
+            observation_dist.batch_shape,
+        )
         batch_shape, time_shape = shape[:-1], shape[-1:]
         event_shape = time_shape + (obs_dim,)
         super().__init__(batch_shape, event_shape, validate_args=validate_args)
@@ -1107,6 +1254,11 @@ class GaussianMRF(TorchDistribution):
         self._init = mvn_to_gaussian(initial_dist)
         self._trans = mvn_to_gaussian(transition_dist)
         self._obs = mvn_to_gaussian(observation_dist)
+        self._support = constraints.independent(observation_dist.support, 1)
+
+    @constraints.dependent_property(event_dim=2)
+    def support(self):
+        return self._support
 
     def expand(self, batch_shape, _instance=None):
         new = self._get_checked_instance(GaussianMRF, _instance)
@@ -1119,8 +1271,11 @@ class GaussianMRF(TorchDistribution):
         new._init = self._init.expand(batch_shape)
         new._trans = self._trans
         new._obs = self._obs
-        super(GaussianMRF, new).__init__(batch_shape, self.event_shape, validate_args=False)
-        new._validate_args = self.__dict__.get('_validate_args')
+        new._support = self._support
+        super(GaussianMRF, new).__init__(
+            batch_shape, self.event_shape, validate_args=False
+        )
+        new._validate_args = self.__dict__.get("_validate_args")
         return new
 
     def log_prob(self, value):
@@ -1130,13 +1285,16 @@ class GaussianMRF(TorchDistribution):
 
         # Combine observation and transition factors.
         logp_oh += self._obs.condition(value).event_pad(left=self.hidden_dim)
-        logp_h += self._obs.marginalize(right=self.obs_dim).event_pad(left=self.hidden_dim)
+        logp_h += self._obs.marginalize(right=self.obs_dim).event_pad(
+            left=self.hidden_dim
+        )
 
         # Concatenate p(obs,hidden) and p(hidden) into a single Gaussian.
         batch_dim = 1 + max(len(self._init.batch_shape) + 1, len(logp_oh.batch_shape))
-        batch_shape = (1,) * (batch_dim - len(logp_oh.batch_shape)) + logp_oh.batch_shape
-        logp = Gaussian.cat([logp_oh.expand(batch_shape),
-                             logp_h.expand(batch_shape)])
+        batch_shape = (1,) * (
+            batch_dim - len(logp_oh.batch_shape)
+        ) + logp_oh.batch_shape
+        logp = Gaussian.cat([logp_oh.expand(batch_shape), logp_h.expand(batch_shape)])
 
         # Eliminate time dimension.
         logp = _sequential_gaussian_tensordot(logp)
