@@ -18,14 +18,20 @@ class InverseGamma(TransformedDistribution):
     :param torch.Tensor concentration: the concentration parameter (i.e. alpha).
     :param torch.Tensor rate: the rate parameter (i.e. beta).
     """
-    arg_constraints = {'concentration': constraints.positive, 'rate': constraints.positive}
+    arg_constraints = {
+        "concentration": constraints.positive,
+        "rate": constraints.positive,
+    }
     support = constraints.positive
     has_rsample = True
 
     def __init__(self, concentration, rate, validate_args=None):
         base_dist = Gamma(concentration, rate)
-        super().__init__(base_dist, PowerTransform(-base_dist.rate.new_ones(())),
-                         validate_args=validate_args)
+        super().__init__(
+            base_dist,
+            PowerTransform(-base_dist.rate.new_ones(())),
+            validate_args=validate_args,
+        )
 
     def expand(self, batch_shape, _instance=None):
         new = self._get_checked_instance(InverseGamma, _instance)
