@@ -5,13 +5,14 @@ import warnings
 from collections import OrderedDict
 
 import torch
-from torch.distributions import constraints
 from torch.distributions.kl import kl_divergence, register_kl
 
 import pyro.distributions.torch
-from pyro.distributions.distribution import Distribution
-from pyro.distributions.score_parts import ScoreParts
-from pyro.distributions.util import broadcast_shape, scale_and_mask
+
+from . import constraints
+from .distribution import Distribution
+from .score_parts import ScoreParts
+from .util import broadcast_shape, scale_and_mask
 
 
 class TorchDistributionMixin(Distribution):
@@ -25,6 +26,7 @@ class TorchDistributionMixin(Distribution):
     :class:`torch.distributions.distribution.Distribution` and then inherit
     from :class:`TorchDistributionMixin`.
     """
+
     def __call__(self, sample_shape=torch.Size()):
         """
         Samples a random value.
@@ -42,7 +44,11 @@ class TorchDistributionMixin(Distribution):
             batched). The shape of the result should be `self.shape()`.
         :rtype: torch.Tensor
         """
-        return self.rsample(sample_shape) if self.has_rsample else self.sample(sample_shape)
+        return (
+            self.rsample(sample_shape)
+            if self.has_rsample
+            else self.sample(sample_shape)
+        )
 
     @property
     def event_dim(self):
@@ -68,6 +74,34 @@ class TorchDistributionMixin(Distribution):
         """
         return sample_shape + self.batch_shape + self.event_shape
 
+    @classmethod
+    def infer_shapes(cls, **arg_shapes):
+        r"""
+        Infers ``batch_shape`` and ``event_shape`` given shapes of args to
+        :meth:`__init__`.
+
+        .. note:: This assumes distribution shape depends only on the shapes
+            of tensor inputs, not in the data contained in those inputs.
+
+        :param \*\*arg_shapes: Keywords mapping name of input arg to
+            :class:`torch.Size` or tuple representing the sizes of each
+            tensor input.
+        :returns: A pair ``(batch_shape, event_shape)`` of the shapes of a
+            distribution that would be created with input args of the given
+            shapes.
+        :rtype: tuple
+        """
+        if cls.support.event_dim > 0:
+            raise NotImplementedError
+        # Assumes distribution is univariate.
+        batch_shapes = []
+        for name, shape in arg_shapes.items():
+            event_dim = cls.arg_constraints.get(name, constraints.real).event_dim
+            batch_shapes.append(shape[: len(shape) - event_dim])
+        batch_shape = torch.Size(broadcast_shape(*batch_shapes))
+        event_shape = torch.Size()
+        return batch_shape, event_shape
+
     def expand(self, batch_shape, _instance=None):
         """
         Returns a new :class:`ExpandedDistribution` instance with batch
@@ -97,13 +131,17 @@ class TorchDistributionMixin(Distribution):
         try:
             expanded_dist = self.expand(torch.Size(sample_shape) + self.batch_shape)
         except NotImplementedError:
-            expanded_dist = TorchDistributionMixin.expand(self, torch.Size(sample_shape) + self.batch_shape)
+            expanded_dist = TorchDistributionMixin.expand(
+                self, torch.Size(sample_shape) + self.batch_shape
+            )
         return expanded_dist
 
     def reshape(self, sample_shape=None, extra_event_dims=None):
-        raise Exception('''
+        raise Exception(
+            """
             .reshape(sample_shape=s, extra_event_dims=n) was renamed and split into
-            .expand_by(sample_shape=s).to_event(reinterpreted_batch_ndims=n).''')
+            .expand_by(sample_shape=s).to_event(reinterpreted_batch_ndims=n)."""
+        )
 
     def to_event(self, reinterpreted_batch_ndims=None):
         """
@@ -150,11 +188,17 @@ class TorchDistributionMixin(Distribution):
         if reinterpreted_batch_ndims == 0:
             return base_dist
         if reinterpreted_batch_ndims < 0:
-            raise ValueError("Cannot remove event dimensions from {}".format(type(self)))
-        return pyro.distributions.torch.Independent(base_dist, reinterpreted_batch_ndims)
+            raise ValueError(
+                "Cannot remove event dimensions from {}".format(type(self))
+            )
+        return pyro.distributions.torch.Independent(
+            base_dist, reinterpreted_batch_ndims
+        )
 
     def independent(self, reinterpreted_batch_ndims=None):
-        warnings.warn("independent is deprecated; use to_event instead", DeprecationWarning)
+        warnings.warn(
+            "independent is deprecated; use to_event instead", DeprecationWarning
+        )
         return self.to_event(reinterpreted_batch_ndims=reinterpreted_batch_ndims)
 
     def mask(self, mask):
@@ -232,6 +276,7 @@ class TorchDistribution(torch.distributions.Distribution, TorchDistributionMixin
     method to improve gradient estimates and set
     ``.has_enumerate_support = True``.
     """
+
     # Provides a default `.expand` method for Pyro distributions which overrides
     # torch.distributions.Distribution.expand (throws a NotImplementedError).
     expand = TorchDistributionMixin.expand
@@ -249,6 +294,7 @@ class MaskedDistribution(TorchDistribution):
     :param mask: A boolean or boolean-valued tensor.
     :type mask: torch.Tensor or bool
     """
+
     arg_constraints = {}
 
     def __init__(self, base_dist, mask):
@@ -271,7 +317,9 @@ class MaskedDistribution(TorchDistribution):
         new._mask = self._mask
         if isinstance(new._mask, torch.Tensor):
             new._mask = new._mask.expand(batch_shape)
-        super(MaskedDistribution, new).__init__(batch_shape, self.event_shape, validate_args=False)
+        super(MaskedDistribution, new).__init__(
+            batch_shape, self.event_shape, validate_args=False
+        )
         new._validate_args = self._validate_args
         return new
 
@@ -295,8 +343,9 @@ class MaskedDistribution(TorchDistribution):
 
     def log_prob(self, value):
         if self._mask is False:
-            shape = broadcast_shape(self.base_dist.batch_shape,
-                                    value.shape[:value.dim() - self.event_dim])
+            shape = broadcast_shape(
+                self.base_dist.batch_shape, value.shape[: value.dim() - self.event_dim]
+            )
             return torch.zeros((), device=value.device).expand(shape)
         if self._mask is True:
             return self.base_dist.log_prob(value)
@@ -324,7 +373,9 @@ class MaskedDistribution(TorchDistribution):
         """
         updated, log_normalizer = self.base_dist.conjugate_update(other)
         updated = updated.mask(self._mask)
-        log_normalizer = torch.where(self._mask, log_normalizer, torch.zeros_like(log_normalizer))
+        log_normalizer = torch.where(
+            self._mask, log_normalizer, torch.zeros_like(log_normalizer)
+        )
         return updated, log_normalizer
 
 
@@ -341,8 +392,9 @@ class ExpandedDistribution(TorchDistribution):
         # Do basic validation. e.g. we should not "unexpand" distributions even if that is possible.
         new_shape, _, _ = self._broadcast_shape(self.batch_shape, batch_shape)
         # Record interstitial and expanded dims/sizes w.r.t. the base distribution
-        new_shape, expanded_sizes, interstitial_sizes = self._broadcast_shape(self.base_dist.batch_shape,
-                                                                              new_shape)
+        new_shape, expanded_sizes, interstitial_sizes = self._broadcast_shape(
+            self.base_dist.batch_shape, new_shape
+        )
         self._batch_shape = new_shape
         self._expanded_sizes = expanded_sizes
         self._interstitial_sizes = interstitial_sizes
@@ -351,8 +403,11 @@ class ExpandedDistribution(TorchDistribution):
     @staticmethod
     def _broadcast_shape(existing_shape, new_shape):
         if len(new_shape) < len(existing_shape):
-            raise ValueError("Cannot broadcast distribution of shape {} to shape {}"
-                             .format(existing_shape, new_shape))
+            raise ValueError(
+                "Cannot broadcast distribution of shape {} to shape {}".format(
+                    existing_shape, new_shape
+                )
+            )
         reversed_shape = list(reversed(existing_shape))
         expanded_sizes, interstitial_sizes = [], []
         for i, size in enumerate(reversed(new_shape)):
@@ -364,9 +419,16 @@ class ExpandedDistribution(TorchDistribution):
                     reversed_shape[i] = size
                     interstitial_sizes.append((-i - 1, size))
             elif reversed_shape[i] != size:
-                raise ValueError("Cannot broadcast distribution of shape {} to shape {}"
-                                 .format(existing_shape, new_shape))
-        return tuple(reversed(reversed_shape)), OrderedDict(expanded_sizes), OrderedDict(interstitial_sizes)
+                raise ValueError(
+                    "Cannot broadcast distribution of shape {} to shape {}".format(
+                        existing_shape, new_shape
+                    )
+                )
+        return (
+            tuple(reversed(reversed_shape)),
+            OrderedDict(expanded_sizes),
+            OrderedDict(interstitial_sizes),
+        )
 
     @property
     def has_rsample(self):
@@ -388,7 +450,9 @@ class ExpandedDistribution(TorchDistribution):
         batch_shape = expanded_sizes + interstitial_sizes
         samples = sample_fn(sample_shape + batch_shape)
         interstitial_idx = len(sample_shape) + len(expanded_sizes)
-        interstitial_sample_dims = tuple(range(interstitial_idx, interstitial_idx + len(interstitial_sizes)))
+        interstitial_sample_dims = tuple(
+            range(interstitial_idx, interstitial_idx + len(interstitial_sizes))
+        )
         for dim1, dim2 in zip(interstitial_dims, interstitial_sample_dims):
             samples = samples.transpose(dim1, dim2)
         return samples.reshape(sample_shape + self.batch_shape + self.event_shape)
@@ -400,12 +464,16 @@ class ExpandedDistribution(TorchDistribution):
         return self._sample(self.base_dist.rsample, sample_shape)
 
     def log_prob(self, value):
-        shape = broadcast_shape(self.batch_shape, value.shape[:value.dim() - self.event_dim])
+        shape = broadcast_shape(
+            self.batch_shape, value.shape[: value.dim() - self.event_dim]
+        )
         log_prob = self.base_dist.log_prob(value)
         return log_prob.expand(shape)
 
     def score_parts(self, value):
-        shape = broadcast_shape(self.batch_shape, value.shape[:value.dim() - self.event_dim])
+        shape = broadcast_shape(
+            self.batch_shape, value.shape[: value.dim() - self.event_dim]
+        )
         log_prob, score_function, entropy_term = self.base_dist.score_parts(value)
         if self.batch_shape != self.base_dist.batch_shape:
             log_prob = log_prob.expand(shape)
@@ -455,7 +523,7 @@ def _kl_masked_masked(p, q):
         mask = p._mask & q._mask
 
     if mask is False:
-        return 0.  # Return a float, since we cannot determine device.
+        return 0.0  # Return a float, since we cannot determine device.
     if mask is True:
         return kl_divergence(p.base_dist, q.base_dist)
     kl = kl_divergence(p.base_dist, q.base_dist)
