@@ -9,18 +9,18 @@ import torch.nn as nn
 import torch.nn.functional as F
 from torch.distributions import Transform, constraints
 
-from pyro.distributions.conditional import ConditionalTransformModule
-from pyro.distributions.torch_transform import TransformModule
-from pyro.distributions.util import copy_docs_from
 from pyro.nn import DenseNN
 
+from ..conditional import ConditionalTransformModule
+from ..torch_transform import TransformModule
+from ..util import copy_docs_from
+
 
 @copy_docs_from(Transform)
 class ConditionedRadial(Transform):
     domain = constraints.real_vector
     codomain = constraints.real_vector
     bijective = True
-    event_dim = 1
 
     def __init__(self, params):
         super().__init__(cache_size=1)
@@ -42,7 +42,9 @@ class ConditionedRadial(Transform):
         :class:`~pyro.distributions.TransformedDistribution` `x` is a sample from the base distribution (or the output
         of a previous transform)
         """
-        x0, alpha_prime, beta_prime = self._params() if callable(self._params) else self._params
+        x0, alpha_prime, beta_prime = (
+            self._params() if callable(self._params) else self._params
+        )
 
         # Ensure invertibility using approach in appendix A.2
         alpha = F.softplus(alpha_prime)
@@ -52,11 +54,13 @@ class ConditionedRadial(Transform):
         diff = x - x0
         r = diff.norm(dim=-1, keepdim=True)
         h = (alpha + r).reciprocal()
-        h_prime = - (h ** 2)
+        h_prime = -(h ** 2)
         beta_h = beta * h
 
-        self._cached_logDetJ = ((x0.size(-1) - 1) * torch.log1p(beta_h) +
-                                torch.log1p(beta_h + beta * h_prime * r)).sum(-1)
+        self._cached_logDetJ = (
+            (x0.size(-1) - 1) * torch.log1p(beta_h)
+            + torch.log1p(beta_h + beta * h_prime * r)
+        ).sum(-1)
         return x + beta_h * diff
 
     def _inverse(self, y):
@@ -69,7 +73,9 @@ class ConditionedRadial(Transform):
         cached on the forward call)
         """
 
-        raise KeyError("ConditionedRadial object expected to find key in intermediates cache but didn't")
+        raise KeyError(
+            "ConditionedRadial object expected to find key in intermediates cache but didn't"
+        )
 
     def log_abs_det_jacobian(self, x, y):
         """
@@ -98,9 +104,6 @@ class Radial(ConditionedRadial, TransformModule):
     :math:`h(\alpha,r)=1/(\alpha+r)`. For this to be an invertible transformation,
     the condition :math:`\beta>-\alpha` is enforced.
 
-    Together with :class:`~pyro.distributions.TransformedDistribution` this provides
-    a way to create richer variational approximations.
-
     Example usage:
 
     >>> base_dist = dist.Normal(torch.zeros(10), torch.ones(10))
@@ -127,14 +130,25 @@ class Radial(ConditionedRadial, TransformModule):
     domain = constraints.real_vector
     codomain = constraints.real_vector
     bijective = True
-    event_dim = 1
 
     def __init__(self, input_dim):
         super().__init__(self._params)
 
-        self.x0 = nn.Parameter(torch.Tensor(input_dim,))
-        self.alpha_prime = nn.Parameter(torch.Tensor(1,))
-        self.beta_prime = nn.Parameter(torch.Tensor(1,))
+        self.x0 = nn.Parameter(
+            torch.Tensor(
+                input_dim,
+            )
+        )
+        self.alpha_prime = nn.Parameter(
+            torch.Tensor(
+                1,
+            )
+        )
+        self.beta_prime = nn.Parameter(
+            torch.Tensor(
+                1,
+            )
+        )
         self.input_dim = input_dim
         self.reset_parameters()
 
@@ -142,7 +156,7 @@ class Radial(ConditionedRadial, TransformModule):
         return self.x0, self.alpha_prime, self.beta_prime
 
     def reset_parameters(self):
-        stdv = 1. / math.sqrt(self.x0.size(0))
+        stdv = 1.0 / math.sqrt(self.x0.size(0))
         self.alpha_prime.data.uniform_(-stdv, stdv)
         self.beta_prime.data.uniform_(-stdv, stdv)
         self.x0.data.uniform_(-stdv, stdv)
@@ -164,16 +178,20 @@ class ConditionalRadial(ConditionalTransformModule):
     For this to be an invertible transformation, the condition :math:`\beta>-\alpha`
     is enforced.
 
-    Together with :class:`~pyro.distributions.TransformedDistribution` this provides
-    a way to create richer variational approximations.
-
     Example usage:
 
-    >>> base_dist = dist.Normal(torch.zeros(10), torch.ones(10))
-    >>> transform = Radial(10)
-    >>> pyro.module("my_transform", transform)  # doctest: +SKIP
-    >>> flow_dist = dist.TransformedDistribution(base_dist, [transform])
-    >>> flow_dist.sample()  # doctest: +SKIP
+    >>> from pyro.nn.dense_nn import DenseNN
+    >>> input_dim = 10
+    >>> context_dim = 5
+    >>> batch_size = 3
+    >>> base_dist = dist.Normal(torch.zeros(input_dim), torch.ones(input_dim))
+    >>> param_dims = [input_dim, 1, 1]
+    >>> hypernet = DenseNN(context_dim, [50, 50], param_dims)
+    >>> transform = ConditionalRadial(hypernet)
+    >>> z = torch.rand(batch_size, context_dim)
+    >>> flow_dist = dist.ConditionalTransformedDistribution(base_dist,
+    ... [transform]).condition(z)
+    >>> flow_dist.sample(sample_shape=torch.Size([batch_size])) # doctest: +SKIP
 
     The inverse of this transform does not possess an analytical solution and is
     left unimplemented. However, the inverse is cached when the forward operation is
@@ -193,7 +211,6 @@ class ConditionalRadial(ConditionalTransformModule):
     domain = constraints.real_vector
     codomain = constraints.real_vector
     bijective = True
-    event_dim = 1
 
     def __init__(self, nn):
         super().__init__()
