@@ -8,22 +8,31 @@ from .haar import HaarReparam
 from .hmm import LinearHMMReparam
 from .loc_scale import LocScaleReparam
 from .neutra import NeuTraReparam
+from .projected_normal import ProjectedNormalReparam
+from .softmax import GumbelSoftmaxReparam
 from .split import SplitReparam
 from .stable import LatentStableReparam, StableReparam, SymmetricStableReparam
+from .strategies import AutoReparam, MinimalReparam
+from .structured import StructuredReparam
 from .studentt import StudentTReparam
 from .transform import TransformReparam
 from .unit_jacobian import UnitJacobianReparam
 
 __all__ = [
+    "AutoReparam",
     "ConjugateReparam",
     "DiscreteCosineReparam",
+    "GumbelSoftmaxReparam",
     "HaarReparam",
     "LatentStableReparam",
     "LinearHMMReparam",
     "LocScaleReparam",
+    "MinimalReparam",
     "NeuTraReparam",
+    "ProjectedNormalReparam",
     "SplitReparam",
     "StableReparam",
+    "StructuredReparam",
     "StudentTReparam",
     "SymmetricStableReparam",
     "TransformReparam",
