@@ -17,19 +17,19 @@ def einsum(equation, *operands):
     """
     # rename symbols to support PyTorch 0.4.1 and earlier,
     # which allow only symbols a-z.
-    symbols = sorted(set(equation) - set(',->'))
-    rename = dict(zip(symbols, 'abcdefghijklmnopqrstuvwxyz'))
-    equation = ''.join(rename.get(s, s) for s in equation)
+    symbols = sorted(set(equation) - set(",->"))
+    rename = dict(zip(symbols, "abcdefghijklmnopqrstuvwxyz"))
+    equation = "".join(rename.get(s, s) for s in equation)
 
-    inputs, output = equation.split('->')
+    inputs, output = equation.split("->")
     if inputs == output:
         return operands[0][...]  # create a new object
-    inputs = inputs.split(',')
+    inputs = inputs.split(",")
 
     shifts = []
     exp_operands = []
     for dims, operand in zip(inputs, operands):
-        shift = operand
+        shift = operand.detach()
         for i, dim in enumerate(dims):
             if dim not in output:
                 shift = shift.max(i, keepdim=True)[0]
@@ -38,8 +38,9 @@ def einsum(equation, *operands):
         exp_operands.append((operand - shift).exp())
 
         # permute shift to match output
-        shift = shift.reshape(torch.Size(size for size, dim in zip(operand.shape, dims)
-                                         if dim in output))
+        shift = shift.reshape(
+            torch.Size(size for size, dim in zip(operand.shape, dims) if dim in output)
+        )
         if shift.dim():
             shift = shift.reshape((1,) * (len(output) - shift.dim()) + shift.shape)
             dims = [dim for dim in dims if dim in output]
