@@ -27,6 +27,7 @@ class Gaussian:
         fast and stable.
     :param torch.Tensor precision: precision matrix of this gaussian.
     """
+
     def __init__(self, log_normalizer, info_vec, precision):
         # NB: using info_vec instead of mean to deal with rank-deficient problem
         assert info_vec.dim() >= 1
@@ -41,9 +42,11 @@ class Gaussian:
 
     @lazy_property
     def batch_shape(self):
-        return broadcast_shape(self.log_normalizer.shape,
-                               self.info_vec.shape[:-1],
-                               self.precision.shape[:-2])
+        return broadcast_shape(
+            self.log_normalizer.shape,
+            self.info_vec.shape[:-1],
+            self.precision.shape[:-2],
+        )
 
     def expand(self, batch_shape):
         n = self.dim()
@@ -76,8 +79,10 @@ class Gaussian:
         """
         if dim < 0:
             dim += len(parts[0].batch_shape)
-        args = [torch.cat([getattr(g, attr) for g in parts], dim=dim)
-                for attr in ["log_normalizer", "info_vec", "precision"]]
+        args = [
+            torch.cat([getattr(g, attr) for g in parts], dim=dim)
+            for attr in ["log_normalizer", "info_vec", "precision"]
+        ]
         return Gaussian(*args)
 
     def event_pad(self, left=0, right=0):
@@ -106,9 +111,11 @@ class Gaussian:
         """
         if isinstance(other, Gaussian):
             assert self.dim() == other.dim()
-            return Gaussian(self.log_normalizer + other.log_normalizer,
-                            self.info_vec + other.info_vec,
-                            self.precision + other.precision)
+            return Gaussian(
+                self.log_normalizer + other.log_normalizer,
+                self.info_vec + other.info_vec,
+                self.precision + other.precision,
+            )
         if isinstance(other, (int, float, torch.Tensor)):
             return Gaussian(self.log_normalizer + other, self.info_vec, self.precision)
         raise ValueError("Unsupported type: {}".format(type(other)))
@@ -175,9 +182,11 @@ class Gaussian:
 
         info_vec = info_a - matvecmul(P_ab, b)
         precision = P_aa
-        log_normalizer = (self.log_normalizer +
-                          -0.5 * matvecmul(P_bb, b).mul(b).sum(-1) +
-                          b.mul(info_b).sum(-1))
+        log_normalizer = (
+            self.log_normalizer
+            + -0.5 * matvecmul(P_bb, b).mul(b).sum(-1)
+            + b.mul(info_b).sum(-1)
+        )
         return Gaussian(log_normalizer, info_vec, precision)
 
     def left_condition(self, value):
@@ -200,8 +209,12 @@ class Gaussian:
         dim = self.dim()
         assert left <= dim
 
-        perm = torch.cat([torch.arange(left, dim, device=value.device),
-                          torch.arange(left, device=value.device)])
+        perm = torch.cat(
+            [
+                torch.arange(left, dim, device=value.device),
+                torch.arange(left, device=value.device),
+            ]
+        )
         return self.event_permute(perm).condition(value)
 
     def marginalize(self, left=0, right=0):
@@ -238,10 +251,12 @@ class Gaussian:
         b_tmp = triangular_solve(info_b.unsqueeze(-1), P_b, upper=False)
         info_vec = info_a - matmul(P_at, b_tmp).squeeze(-1)
 
-        log_normalizer = (self.log_normalizer +
-                          0.5 * n_b * math.log(2 * math.pi) -
-                          P_b.diagonal(dim1=-2, dim2=-1).log().sum(-1) +
-                          0.5 * b_tmp.squeeze(-1).pow(2).sum(-1))
+        log_normalizer = (
+            self.log_normalizer
+            + 0.5 * n_b * math.log(2 * math.pi)
+            - P_b.diagonal(dim1=-2, dim2=-1).log().sum(-1)
+            + 0.5 * b_tmp.squeeze(-1).pow(2).sum(-1)
+        )
         return Gaussian(log_normalizer, info_vec, precision)
 
     def event_logsumexp(self):
@@ -250,10 +265,16 @@ class Gaussian:
         """
         n = self.dim()
         chol_P = cholesky(self.precision)
-        chol_P_u = triangular_solve(self.info_vec.unsqueeze(-1), chol_P, upper=False).squeeze(-1)
+        chol_P_u = triangular_solve(
+            self.info_vec.unsqueeze(-1), chol_P, upper=False
+        ).squeeze(-1)
         u_P_u = chol_P_u.pow(2).sum(-1)
-        return (self.log_normalizer + 0.5 * n * math.log(2 * math.pi) + 0.5 * u_P_u -
-                chol_P.diagonal(dim1=-2, dim2=-1).log().sum(-1))
+        return (
+            self.log_normalizer
+            + 0.5 * n * math.log(2 * math.pi)
+            + 0.5 * u_P_u
+            - chol_P.diagonal(dim1=-2, dim2=-1).log().sum(-1)
+        )
 
 
 class AffineNormal:
@@ -275,6 +296,7 @@ class AffineNormal:
     :param torch.Tensor scale: Standard deviation for ``Y``.
         Should have rightmost shape ``(y_dim,)``.
     """
+
     def __init__(self, matrix, loc, scale):
         assert loc.shape == scale.shape
         assert matrix.shape[:-2] == loc.shape[:-1]
@@ -294,8 +316,11 @@ class AffineNormal:
             precision = matmul(prec_sqrt, prec_sqrt.transpose(-1, -2))
             delta = (value - self.loc) / self.scale
             info_vec = matvecmul(prec_sqrt, delta)
-            log_normalizer = (-0.5 * self.loc.size(-1) * math.log(2 * math.pi)
-                              - 0.5 * delta.pow(2).sum(-1) - self.scale.log().sum(-1))
+            log_normalizer = (
+                -0.5 * self.loc.size(-1) * math.log(2 * math.pi)
+                - 0.5 * delta.pow(2).sum(-1)
+                - self.scale.log().sum(-1)
+            )
             return Gaussian(log_normalizer, info_vec, precision)
         else:
             return self.to_gaussian().condition(value)
@@ -327,7 +352,8 @@ class AffineNormal:
     def to_gaussian(self):
         if self._gaussian is None:
             mvn = torch.distributions.Independent(
-                torch.distributions.Normal(self.loc, scale=self.scale), 1)
+                torch.distributions.Normal(self.loc, scale=self.scale), 1
+            )
             y_gaussian = mvn_to_gaussian(mvn)
             self._gaussian = _matrix_and_gaussian_to_gaussian(self.matrix, y_gaussian)
         return self._gaussian
@@ -376,9 +402,10 @@ def mvn_to_gaussian(mvn):
     :return: An equivalent Gaussian object.
     :rtype: ~pyro.ops.gaussian.Gaussian
     """
-    assert (isinstance(mvn, torch.distributions.MultivariateNormal) or
-            (isinstance(mvn, torch.distributions.Independent) and
-             isinstance(mvn.base_dist, torch.distributions.Normal)))
+    assert isinstance(mvn, torch.distributions.MultivariateNormal) or (
+        isinstance(mvn, torch.distributions.Independent)
+        and isinstance(mvn.base_dist, torch.distributions.Normal)
+    )
     if isinstance(mvn, torch.distributions.Independent):
         mvn = mvn.base_dist
         precision_diag = mvn.scale.pow(-2)
@@ -391,9 +418,11 @@ def mvn_to_gaussian(mvn):
         scale_diag = mvn.scale_tril.diagonal(dim1=-2, dim2=-1)
 
     n = mvn.loc.size(-1)
-    log_normalizer = (-0.5 * n * math.log(2 * math.pi) +
-                      -0.5 * (info_vec * mvn.loc).sum(-1) -
-                      scale_diag.log().sum(-1))
+    log_normalizer = (
+        -0.5 * n * math.log(2 * math.pi)
+        + -0.5 * (info_vec * mvn.loc).sum(-1)
+        - scale_diag.log().sum(-1)
+    )
     return Gaussian(log_normalizer, info_vec, precision)
 
 
@@ -403,8 +432,9 @@ def _matrix_and_gaussian_to_gaussian(matrix, y_gaussian):
     P_xy = -neg_P_xy
     P_yx = P_xy.transpose(-1, -2)
     P_xx = matmul(neg_P_xy, matrix.transpose(-1, -2))
-    precision = torch.cat([torch.cat([P_xx, P_xy], -1),
-                           torch.cat([P_yx, P_yy], -1)], -2)
+    precision = torch.cat(
+        [torch.cat([P_xx, P_xy], -1), torch.cat([P_yx, P_yy], -1)], -2
+    )
     info_y = y_gaussian.info_vec
     info_x = -matvecmul(matrix, info_y)
     info_vec = torch.cat([info_x, info_y], -1)
@@ -425,9 +455,10 @@ def matrix_and_mvn_to_gaussian(matrix, mvn):
     :return: A Gaussian with broadcasted batch shape and ``.dim() == x_dim + y_dim``.
     :rtype: ~pyro.ops.gaussian.Gaussian
     """
-    assert (isinstance(mvn, torch.distributions.MultivariateNormal) or
-            (isinstance(mvn, torch.distributions.Independent) and
-             isinstance(mvn.base_dist, torch.distributions.Normal)))
+    assert isinstance(mvn, torch.distributions.MultivariateNormal) or (
+        isinstance(mvn, torch.distributions.Independent)
+        and isinstance(mvn.base_dist, torch.distributions.Normal)
+    )
     assert isinstance(matrix, torch.Tensor)
     x_dim, y_dim = matrix.shape[-2:]
     assert mvn.event_shape == (y_dim,)
@@ -468,8 +499,16 @@ def gaussian_tensordot(x, y, dims=0):
     assert nb >= 0
     assert nc >= 0
 
-    Paa, Pba, Pbb = x.precision[..., :na, :na], x.precision[..., na:, :na], x.precision[..., na:, na:]
-    Qbb, Qbc, Qcc = y.precision[..., :nb, :nb], y.precision[..., :nb, nb:], y.precision[..., nb:, nb:]
+    Paa, Pba, Pbb = (
+        x.precision[..., :na, :na],
+        x.precision[..., na:, :na],
+        x.precision[..., na:, na:],
+    )
+    Qbb, Qbc, Qcc = (
+        y.precision[..., :nb, :nb],
+        y.precision[..., :nb, nb:],
+        y.precision[..., nb:, nb:],
+    )
     xa, xb = x.info_vec[..., :na], x.info_vec[..., na:]  # x.precision @ x.mean
     yb, yc = y.info_vec[..., :nb], y.info_vec[..., nb:]  # y.precision @ y.mean
 
@@ -491,7 +530,11 @@ def gaussian_tensordot(x, y, dims=0):
         if na + nc > 0:
             info_vec = info_vec - matmul(LinvBt, Linvb).squeeze(-1)
         logdet = torch.diagonal(L, dim1=-2, dim2=-1).log().sum(-1)
-        diff = 0.5 * nb * math.log(2 * math.pi) + 0.5 * Linvb.squeeze(-1).pow(2).sum(-1) - logdet
+        diff = (
+            0.5 * nb * math.log(2 * math.pi)
+            + 0.5 * Linvb.squeeze(-1).pow(2).sum(-1)
+            - logdet
+        )
         log_normalizer = log_normalizer + diff
 
     return Gaussian(log_normalizer, info_vec, precision)
