@@ -14,7 +14,9 @@ def hessian(y, xs):
     flat_dy = torch.cat([dy.reshape(-1) for dy in dys])
     H = []
     for dyi in flat_dy:
-        Hi = torch.cat([Hij.reshape(-1) for Hij in torch.autograd.grad(dyi, xs, retain_graph=True)])
+        Hi = torch.cat(
+            [Hij.reshape(-1) for Hij in torch.autograd.grad(dyi, xs, retain_graph=True)]
+        )
         H.append(Hi)
     H = torch.stack(H)
     return H
