@@ -4,11 +4,25 @@
 import math
 
 import torch
-
+from torch.fft import irfft, rfft
 
 _ROOT_TWO_INVERSE = 1.0 / math.sqrt(2.0)
 
 
+def as_complex(x):
+    """
+    Similar to :func:`torch.view_as_complex` but copies data in case strides
+    are not multiples of two.
+    """
+    if any(stride % 2 for stride in x.stride()[:-1]):
+        # First try to normalize strides.
+        x = x.squeeze().reshape(x.shape)
+    if any(stride % 2 for stride in x.stride()[:-1]):
+        # Fall back to copying data.
+        x = x.clone()
+    return torch.view_as_complex(x)
+
+
 def block_diag_embed(mat):
     """
     Takes a tensor of shape (..., B, M, N) and returns a block diagonal tensor
@@ -39,7 +53,7 @@ def block_diagonal(mat, block_size):
     mat = mat.reshape(mat.shape[:-2] + (B, M, B, N))
     mat = mat.transpose(-2, -3)
     mat = mat.reshape(mat.shape[:-4] + (B * B, M, N))
-    return mat[..., ::B + 1, :, :]
+    return mat[..., :: B + 1, :, :]
 
 
 def periodic_repeat(tensor, size, dim):
@@ -103,7 +117,9 @@ def periodic_cumsum(tensor, period, dim):
         tensor = torch.nn.functional.pad(tensor, (0, 0) * (-1 - dim) + (0, padding))
 
     # Accumulate.
-    shape = tensor.shape[:dim] + (repeats, period) + tensor.shape[tensor.dim() + dim + 1:]
+    shape = (
+        tensor.shape[:dim] + (repeats, period) + tensor.shape[tensor.dim() + dim + 1 :]
+    )
     result = tensor.reshape(shape).cumsum(dim=dim - 1).reshape(tensor.shape)
 
     # Truncate to original size.
@@ -147,7 +163,9 @@ def periodic_features(duration, max_period=None, min_period=None, **options):
 
     t = torch.arange(float(duration), **options).unsqueeze(-1).unsqueeze(-1)
     phase = torch.tensor([0, math.pi / 2], **options).unsqueeze(-1)
-    freq = torch.arange(1, max_period / min_period, **options).mul_(2 * math.pi / max_period)
+    freq = torch.arange(1, max_period / min_period, **options).mul_(
+        2 * math.pi / max_period
+    )
     result = (freq * t + phase).cos_().reshape(duration, -1).contiguous()
     return result
 
@@ -183,13 +201,7 @@ def next_fast_len(size):
         next_size += 1
 
 
-def _complex_mul(a, b):
-    ar, ai = a.unbind(-1)
-    br, bi = b.unbind(-1)
-    return torch.stack([ar * br - ai * bi, ar * bi + ai * br], dim=-1)
-
-
-def convolve(signal, kernel, mode='full'):
+def convolve(signal, kernel, mode="full"):
     """
     Computes the 1-d convolution of signal by kernel using FFTs.
     The two arguments should have the same rightmost dim, but may otherwise be
@@ -207,26 +219,26 @@ def convolve(signal, kernel, mode='full'):
     """
     m = signal.size(-1)
     n = kernel.size(-1)
-    if mode == 'full':
+    if mode == "full":
         truncate = m + n - 1
-    elif mode == 'valid':
+    elif mode == "valid":
         truncate = max(m, n) - min(m, n) + 1
-    elif mode == 'same':
+    elif mode == "same":
         truncate = max(m, n)
     else:
-        raise ValueError('Unknown mode: {}'.format(mode))
+        raise ValueError("Unknown mode: {}".format(mode))
 
     # Compute convolution using fft.
     padded_size = m + n - 1
     # Round up for cheaper fft.
     fast_ftt_size = next_fast_len(padded_size)
-    f_signal = torch.rfft(torch.nn.functional.pad(signal, (0, fast_ftt_size - m)), 1, onesided=False)
-    f_kernel = torch.rfft(torch.nn.functional.pad(kernel, (0, fast_ftt_size - n)), 1, onesided=False)
-    f_result = _complex_mul(f_signal, f_kernel)
-    result = torch.irfft(f_result, 1, onesided=False)
+    f_signal = rfft(signal, n=fast_ftt_size)
+    f_kernel = rfft(kernel, n=fast_ftt_size)
+    f_result = f_signal * f_kernel
+    result = irfft(f_result, n=fast_ftt_size)
 
     start_idx = (padded_size - truncate) // 2
-    return result[..., start_idx: start_idx + truncate]
+    return result[..., start_idx : start_idx + truncate]
 
 
 def repeated_matmul(M, n):
@@ -239,7 +251,9 @@ def repeated_matmul(M, n):
     :param int n: The order of the largest product :math:`M^n`
     :returns torch.Tensor: A batch of square tensors of shape (n, ..., N, N)
     """
-    assert M.size(-1) == M.size(-2), "Input tensors must satisfy M.size(-1) == M.size(-2)."
+    assert M.size(-1) == M.size(
+        -2
+    ), "Input tensors must satisfy M.size(-1) == M.size(-2)."
     assert n > 0, "argument n to parallel_scan_repeated_matmul must be 1 or larger"
 
     doubling_rounds = 0 if n <= 2 else math.ceil(math.log(n, 2)) - 1
@@ -256,12 +270,6 @@ def repeated_matmul(M, n):
     return result[0:n]
 
 
-def _real_of_complex_mul(a, b):
-    ar, ai = a.unbind(-1)
-    br, bi = b.unbind(-1)
-    return ar * br - ai * bi
-
-
 def dct(x, dim=-1):
     """
     Discrete cosine transform of type II, scaled to be orthonormal.
@@ -276,7 +284,7 @@ def dct(x, dim=-1):
     if dim >= 0:
         dim -= x.dim()
     if dim != -1:
-        y = x.reshape(x.shape[:dim + 1] + (-1,)).transpose(-1, -2)
+        y = x.reshape(x.shape[: dim + 1] + (-1,)).transpose(-1, -2)
         return dct(y).transpose(-1, -2).reshape(x.shape)
 
     # Ref: http://fourier.eng.hmc.edu/e161/lectures/dct/node2.html
@@ -284,13 +292,22 @@ def dct(x, dim=-1):
     # Step 1
     y = torch.cat([x[..., ::2], x[..., 1::2].flip(-1)], dim=-1)
     # Step 2
-    Y = torch.rfft(y, 1, onesided=False)
+    Y = rfft(y, n=N)
     # Step 3
-    coef_real = torch.cos(torch.linspace(0, 0.5 * math.pi, N + 1, dtype=x.dtype, device=x.device))
-    coef = torch.stack([coef_real[:-1], -coef_real[1:].flip(-1)], dim=-1)
-    X = _real_of_complex_mul(coef, Y)
+    coef_real = torch.cos(
+        torch.linspace(0, 0.5 * math.pi, N + 1, dtype=x.dtype, device=x.device)
+    )
+    M = Y.size(-1)
+    coef = torch.stack([coef_real[:M], -coef_real[-M:].flip(-1)], dim=-1)
+    X = as_complex(coef) * Y
+    # NB: if we use the full-length version Y_full = fft(y, n=N), then
+    # the real part of the later half of X will be the flip
+    # of the negative of the imaginary part of the first half
+    X = torch.cat([X.real, -X.imag[..., 1 : (N - M + 1)].flip(-1)], dim=-1)
     # orthogonalize
-    scale = torch.cat([x.new_tensor([math.sqrt(N)]), x.new_full((N - 1,), math.sqrt(0.5 * N))])
+    scale = torch.cat(
+        [x.new_tensor([math.sqrt(N)]), x.new_full((N - 1,), math.sqrt(0.5 * N))]
+    )
     return X / scale
 
 
@@ -308,11 +325,13 @@ def idct(x, dim=-1):
     if dim >= 0:
         dim -= x.dim()
     if dim != -1:
-        y = x.reshape(x.shape[:dim + 1] + (-1,)).transpose(-1, -2)
+        y = x.reshape(x.shape[: dim + 1] + (-1,)).transpose(-1, -2)
         return idct(y).transpose(-1, -2).reshape(x.shape)
 
     N = x.size(-1)
-    scale = torch.cat([x.new_tensor([math.sqrt(N)]), x.new_full((N - 1,), math.sqrt(0.5 * N))])
+    scale = torch.cat(
+        [x.new_tensor([math.sqrt(N)]), x.new_full((N - 1,), math.sqrt(0.5 * N))]
+    )
     x = x * scale
     # Step 1, solve X = cos(k) * Yr + sin(k) * Yi
     # We know that Y[1:] is conjugate to Y[:0:-1], hence
@@ -321,14 +340,16 @@ def idct(x, dim=-1):
     # and Yi[1:] = sin(k) * X[1:] - cos(k) * X[:0:-1]
     # In addition, Yi[0] = 0, Yr[0] = X[0]
     # In other words, Y = complex_mul(e^ik, X - i[0, X[:0:-1]])
-    xi = torch.nn.functional.pad(-x[..., 1:], (0, 1)).flip(-1)
-    X = torch.stack([x, xi], dim=-1)
-    coef_real = torch.cos(torch.linspace(0, 0.5 * math.pi, N + 1))
-    coef = torch.stack([coef_real[:-1], coef_real[1:].flip(-1)], dim=-1)
-    half_size = N // 2 + 1
-    Y = _complex_mul(coef[..., :half_size, :], X[..., :half_size, :])
+    M = N // 2 + 1  # half size
+    xi = torch.nn.functional.pad(-x[..., N - M + 1 :], (0, 1)).flip(-1)
+    X = torch.stack([x[..., :M], xi], dim=-1)
+    coef_real = torch.cos(
+        torch.linspace(0, 0.5 * math.pi, N + 1, dtype=x.dtype, device=x.device)
+    )
+    coef = torch.stack([coef_real[:M], coef_real[-M:].flip(-1)], dim=-1)
+    Y = as_complex(coef) * as_complex(X)
     # Step 2
-    y = torch.irfft(Y, 1, onesided=True, signal_sizes=(N,))
+    y = irfft(Y, n=N)
     # Step 3
     return torch.stack([y, y.flip(-1)], axis=-1).reshape(x.shape[:-1] + (-1,))[..., :N]
 
@@ -344,7 +365,7 @@ def haar_transform(x):
     :rtype: Tensor
     """
     n = x.size(-1) // 2
-    even, odd, end = x[..., 0:n+n:2], x[..., 1:n+n:2], x[..., n+n:]
+    even, odd, end = x[..., 0 : n + n : 2], x[..., 1 : n + n : 2], x[..., n + n :]
     hi = _ROOT_TWO_INVERSE * (even - odd)
     lo = _ROOT_TWO_INVERSE * (even + odd)
     if n >= 2:
@@ -362,7 +383,7 @@ def inverse_haar_transform(x):
     :rtype: Tensor
     """
     n = x.size(-1) // 2
-    lo, hi, end = x[..., :n], x[..., n:n+n], x[..., n+n:]
+    lo, hi, end = x[..., :n], x[..., n : n + n], x[..., n + n :]
     if n >= 2:
         lo = inverse_haar_transform(lo)
     even = _ROOT_TWO_INVERSE * (lo + hi)
@@ -375,7 +396,7 @@ def inverse_haar_transform(x):
 def cholesky(x):
     if x.size(-1) == 1:
         return x.sqrt()
-    return x.cholesky()
+    return torch.linalg.cholesky(x)
 
 
 def cholesky_solve(x, y):
@@ -403,8 +424,27 @@ def triangular_solve(x, y, upper=False, transpose=False):
 
 
 def precision_to_scale_tril(P):
-    Lf = torch.cholesky(torch.flip(P, (-2, -1)))
+    Lf = torch.linalg.cholesky(torch.flip(P, (-2, -1)))
     L_inv = torch.transpose(torch.flip(Lf, (-2, -1)), -2, -1)
-    L = torch.triangular_solve(torch.eye(P.shape[-1], dtype=P.dtype, device=P.device),
-                               L_inv, upper=False)[0]
+    L = torch.triangular_solve(
+        torch.eye(P.shape[-1], dtype=P.dtype, device=P.device), L_inv, upper=False
+    )[0]
     return L
+
+
+def safe_normalize(x, *, p=2):
+    """
+    Safely project a vector onto the sphere wrt the ``p``-norm. This avoids the
+    singularity at zero by mapping zero to the vector ``[1, 0, 0, ..., 0]``.
+
+    :param torch.Tensor x: A vector
+    :param float p: The norm exponent, defaults to 2 i.e. the Euclidean norm.
+    :returns: A normalized version ``x / ||x||_p``.
+    :rtype: Tensor
+    """
+    assert isinstance(p, (float, int))
+    assert p >= 0
+    norm = torch.linalg.norm(x, dim=-1, ord=p, keepdim=True)
+    x = x / norm.clamp(min=torch.finfo(x.dtype).tiny)
+    x.data[..., 0][x.data.eq(0).all(dim=-1)] = 1  # Avoid the singularity.
+    return x
