@@ -2,6 +2,7 @@
 # SPDX-License-Identifier: Apache-2.0
 
 import pyro.optim.multi  # noqa F403
+from pyro.optim.horovod import HorovodOptimizer
 from pyro.optim.lr_scheduler import PyroLRScheduler
 from pyro.optim.optim import AdagradRMSProp, ClippedAdam, DCTAdam, PyroOptim
 from pyro.optim.pytorch_optimizers import *  # noqa F403
@@ -11,6 +12,7 @@ __all__ = [
     "AdagradRMSProp",
     "ClippedAdam",
     "DCTAdam",
+    "HorovodOptimizer",
     "PyroOptim",
     "PyroLRScheduler",
 ]
