@@ -1,20 +1,41 @@
 # Copyright (c) 2017-2019 Uber Technologies, Inc.
 # SPDX-License-Identifier: Apache-2.0
 
-from .handlers import (block, broadcast, condition, do, enum, escape, infer_config, lift, markov, mask, queue, reparam,
-                       replay, scale, seed, trace, uncondition)
-from .runtime import NonlocalExit
+from .handlers import (
+    block,
+    broadcast,
+    collapse,
+    condition,
+    do,
+    enum,
+    escape,
+    infer_config,
+    lift,
+    markov,
+    mask,
+    queue,
+    reparam,
+    replay,
+    scale,
+    seed,
+    trace,
+    uncondition,
+)
+from .messenger import unwrap
+from .runtime import NonlocalExit, get_mask
 from .trace_struct import Trace
 from .util import enable_validation, is_validation_enabled
 
 __all__ = [
     "block",
     "broadcast",
+    "collapse",
     "condition",
     "do",
     "enable_validation",
     "enum",
     "escape",
+    "get_mask",
     "infer_config",
     "is_validation_enabled",
     "lift",
@@ -29,4 +50,5 @@ __all__ = [
     "trace",
     "Trace",
     "uncondition",
+    "unwrap",
 ]
